#!/v/guest/sw/bin/perl -w

###############################################################################
##                                                                           ##
##    Copyright (c) 2006 - 2009 by Steffen Beyer.                            ##
##    All rights reserved.                                                   ##
##                                                                           ##
##    This program is free software; you can redistribute it                 ##
##    and/or modify it under the same terms as Perl itself.                  ##
##                                                                           ##
###############################################################################

use strict;

use lib
qw(
    /u/sb/sw/pkg/perl/lib/perl/5.10.1/amd64-freebsd
    /u/sb/sw/pkg/perl/lib/perl/5.10.1
    /u/sb/sw/pkg/perl/lib/perl/site_perl/5.10.1/amd64-freebsd
    /u/sb/sw/pkg/perl/lib/perl/site_perl/5.10.1
);

use Date::Calc qw(:all);
use Storable qw(lock_nstore lock_retrieve);
use Fcntl qw(:flock);

umask(022);

my($Title)   = 'Sports Club Events Calendar';
my($Conf)    = 'members.txt';
my($File)    = 'data.bin';
my($Dots)    = '...';
my($Data)    = {};
my($Time)    = time();
my(@Today)   = Today();
my($Today)   = Date_to_Days(@Today);
my(@Date)    = @Today;
my($Lang)    = 1;
my($Save)    = 0;
my($Event)   = '';
my($Person)  = '';
my($Command) = '';
my($Timeout) = 0;
my($Updated) = 0;
my(@Members) = ($Dots);
my(@Fixed)   = ();
my(@Text)    = ();
my(@html)    = ();
my(@lang)    = ();
my(@Nth)     = ();

my($c,$e,$k,$l,$m,$n,$o,$p,$z,$fixed);

###########################################################################################
#                                                                                         #
# Fixed recurring events, either on a fixed day of week, or on the "nth" day of week:     #
#                                                                                         #
# 1st index:       1=Monday 2=Tuesday 3=Wednesday 4=Thursday 5=Friday 6=Saturday 7=Sunday #
# 2nd index: 0=Any 1=First  2=Second  3=Third     4=Fourth   5=Last                       #
#                                                                                         #
# If there are several events on the same day at different times,                         #
# define them in reverse order to make them appear in chronological order:                #
#                                                                                         #
###########################################################################################

$Fixed[1][0] = [qq{19:30h - 21:00h: Regularly Recurring Event: Training Session on Monday\n}];
$Fixed[4][0] = [qq{18:30h - 20:00h: Regularly Recurring Event: Playing Session on Thursday\n}];
$Fixed[7][5] =
[
    <<'KOOPZONDAG'
Koopzondag Den Haag
In stadsdeel Escamp zijn de winkels iedere laatste zondag van de maand open
KOOPZONDAG
];
$Fixed[2][1] =
[
    <<'AMS.PM.2',
[English version follows the dutch text]

De huidige bijeenkomst vindt plaats om 20:00 tot 22:30 uur
op het kantoor van XS4All, Eekholt 42, Diemen.

Voor een routebeschrijving, zie
http://www.xs4all.nl/overxs4all/contact/route.php

Amsterdam.pm staat voor de "Amsterdamse Perl Mongers", een groep van
gebruikers van Perl. In tegenstelling tot wat de naam suggereert is
Amsterdam.pm niet beperkt tot alleen Amsterdam, maar functioneert ook
als de algemene Nederlandse gebruikersgroep. Iedereen die gelegenheid
heeft is welkom, ook al bent u al bij Alphen.pm, Nijmegen.pm,
Rotterdam.pm, Echt.pm, Exloo.pm, Groningen.pm of Wageningen.pm op
bezoek geweest.

Amsterdam.pm organiseert informele bijeenkomsten waar Perl gebruikers
kunnen samenkomen en informatie en gebruikservaringen met betrekking
tot Perl kunnen uitwisselen.

De bijeenkomsten vinden normaliter plaats op elke eerste dinsdag van
de maand. Zoals gebruikelijk voor dit soort bijeenkomsten is er ook
hier gelegenheid tot CAcert certificering en het uitwisselen van PGP
keys. De voertaal binnen Amsterdam.pm is in principe Nederlands, maar
indien nodig zal Engels worden gebruikt, b.v. om te communiceren met
niet-Nederlandssprekende aanwezigen.

Bezoek onze Web site http://amsterdam.pm.org voor meer details.

[English version]

Our meeting today is from 20:00 till 22:30 o'clock
at the office of XS4All, Eekholt 42, Diemen.

See http://www.xs4all.nl/en/overxs4all/contact/route.php  for directions
how to get there.

Amsterdam.pm stands for the Amsterdam Perl Mongers. We're basically a
Perl user group. Despite its name, it is not local to the Amsterdam
environment, but it welcomes Perl mongers from all over the
Netherlands.

Amsterdam.pm organises informal meetings where Perl users can meet,
and exchange information and experiences with regard to using Perl.

The meetings are normally held every first Tuesday of the month. As
conventional for these kind of meetings, there is also an opportunity
to get CAcert certified, and to exchange PGP keys. Although the
preferred language for communication is Dutch, English will be spoken
if necessary.

See http://amsterdam.pm.org for more details.
AMS.PM.2
    <<'AMS.PM.1'
[English version follows the dutch text]

Liefhebbers van een etentje vooraf kunnen tussen 17:45 en 18:15 uur
verzamelen. Om 18:15 (&eacute;cht om 18:15!) uur gaan we naar restaurant
Ling Hong in Diemen (zie b.v. http://restaurant.diemen.klup.nl/) om een
hapje te eten.

[English version]

Should you want to join some of us for dinner, please gather between
17:45 and 18:15 h. At 18:15 h sharp we'll leave for restaurant Ling Hong
in Diemen (http://www.linghong.tk) for dinner.
AMS.PM.1
];

my($NAME) = 0;
my($TIME) = 1;
my($LIST) = 2;

$Text[0][1] = "Please select your preferred language:";
$Text[0][2] = "S.v.p. s&eacute;lectionnez votre langue pr&eacute;f&eacute;r&eacute;e:";
$Text[0][3] = "Bitte w&auml;hlen Sie Ihre bevorzugte Sprache:";
$Text[0][4] = "Por favor seleccione su idioma preferido:";
$Text[0][5] = "Por favor selecione sua l&iacute;ngua preferida:";
$Text[0][6] = "U meest gewenste taal kiezen a.u.b.:";

$Text[1][1] = "Update";
$Text[1][2] = "Actualiser";
$Text[1][3] = "Aktualisieren";
$Text[1][4] = "Actualizar";
$Text[1][5] = "Atualizar";
$Text[1][6] = "Actualiseren";

$Text[2][1] = "There are no events for this date yet.";
$Text[2][2] = "Il n'y a pas encore d'&eacute;v&eacute;nements pour cette date.";
$Text[2][3] = "Es gibt noch keine Veranstaltungen zu diesem Datum.";
$Text[2][4] = "Todavia no hay eventos para esta fecha.";
$Text[2][5] = "Ainda n&atilde;o tem eventos para esta data.";
$Text[2][6] = "Er zijn nog geen evenementen voor dit datum.";

$Text[3][1] = "There is one event:";
$Text[3][2] = "Il y a un &eacute;v&eacute;nement:";
$Text[3][3] = "Es gibt eine Veranstaltung:";
$Text[3][4] = "Hay un evento:";
$Text[3][5] = "Tem um evento:";
$Text[3][6] = "Er is een evenement:";

$Text[4][1] = "There are # events:";
$Text[4][2] = "Il y a # &eacute;v&eacute;nements:";
$Text[4][3] = "Es gibt # Veranstaltungen:";
$Text[4][4] = "Hay # eventos:";
$Text[4][5] = "Tem # eventos:";
$Text[4][6] = "Er zijn # evenementen:";

$Text[5][1] = "There are no participants for this event yet.";
$Text[5][2] = "Il n'y a pas encore de participants pour cet &eacute;v&eacute;nement.";
$Text[5][3] = "Es gibt noch keine Teilnehmer zu dieser Veranstaltung.";
$Text[5][4] = "Todavia no hay participantes para este evento.";
$Text[5][5] = "Ainda n&atilde;o tem participantes para este evento.";
$Text[5][6] = "Er zijn nog geen deelnemers voor dit evenement.";

$Text[6][1] = "There is one participant:";
$Text[6][2] = "Il y a un participant:";
$Text[6][3] = "Es gibt einen Teilnehmer:";
$Text[6][4] = "Hay un participante:";
$Text[6][5] = "Tem um participante:";
$Text[6][6] = "Er is een deelnemer:";

$Text[7][1] = "There are # participants:";
$Text[7][2] = "Il y a # participants:";
$Text[7][3] = "Es gibt # Teilnehmer:";
$Text[7][4] = "Hay # participantes:";
$Text[7][5] = "Tem # participantes:";
$Text[7][6] = "Er zijn # deelnemers:";

$Text[8][1] = "Delete this event";
$Text[8][2] = "&Eacute;ffacer cet &eacute;v&eacute;nement";
$Text[8][3] = "Diese Veranstaltung l&ouml;schen";
$Text[8][4] = "Borrar este evento";
$Text[8][5] = "Apagar este evento";
$Text[8][6] = "Dit evenement wissen";

$Text[9][1] = "Remove this participant";
$Text[9][2] = "Retirer ce participant";
$Text[9][3] = "Diesen Teilnehmer entfernen";
$Text[9][4] = "Quitar este participante";
$Text[9][5] = "Tirar este participante";
$Text[9][6] = "Deze deelnemer verwijderen";

$Text[10][1] = "Add frequent player";
$Text[10][2] = "Ajouter joueur fr&eacute;quent";
$Text[10][3] = "H&auml;ufigen Mitspieler hinzuf&uuml;gen";
$Text[10][4] = "Adicionar jugador frecuente";
$Text[10][5] = "Acrescentar jogador freq&uuml;ente";
$Text[10][6] = "Vaake medespeler toevoegen";

$Text[11][1] = "Add other participant";
$Text[11][2] = "Ajouter autre participant";
$Text[11][3] = "Anderen Teilnehmer hinzuf&uuml;gen";
$Text[11][4] = "Adicionar otro participante";
$Text[11][5] = "Acrescentar outro participante";
$Text[11][6] = "Andere deelnemer toevoegen";

$Text[12][1] = "Add new event";
$Text[12][2] = "Ajouter nouvel &eacute;v&eacute;nement";
$Text[12][3] = "Neue Veranstaltung hinzuf&uuml;gen";
$Text[12][4] = "Adicionar nuevo evento";
$Text[12][5] = "Acrescentar novo evento";
$Text[12][6] = "Nieuwe evenement toevoegen";

if (-e $Conf)
{
    eval
    {
        open(CONF, "<$Conf") or die;
        flock(CONF, LOCK_SH) or die;
        $Updated = (stat(CONF))[9]||0;
        seek(CONF, 0, 0) or die;
        @Members = sort grep { !/^\.*$/ } map { my($x) = $_; $x =~ s/^\s+//; $x =~ s/\s+$//; $x =~ s/\#.*$//; $x } <CONF>;
        flock(CONF, LOCK_UN) or die;
        close(CONF) or die;
        unshift(@Members, $Dots);
    };
    if ($@) { @Members = ($Dots); $Updated = 0; }
}

if (-e $File)
{
    eval { $Data = lock_retrieve($File); };
    if ($@) { $Data = {}; }
}

$m = $Today - 30; # purge events older than 30 days
foreach $n (keys %{$Data})
{
    if ($n < $m) { delete $Data->{$n}; $Save = 1; }
}

init_tables();
process_query_string();

my(@Prev) = Add_Delta_YM(@Date,0,-1);
my(@Next) = Add_Delta_YM(@Date,0,+1);
my($Head) = html(Date_to_Text_Long(@Date,$Lang));
my($Key)  = Date_to_Days(@Date);
my($Dow)  = Day_of_Week(@Date);

execute_command();

if ($Key >= $Today)
{
    @Nth = ();
    foreach $n (nth_dow($Dow,@Date))
    {
        push(@Nth,$n) if (defined $Fixed[$Dow][$n] and ref($Fixed[$Dow][$n]) eq 'ARRAY');
    }
    if (@Nth)
    {
        foreach $k (@Nth)
        {
            foreach $fixed (@{$Fixed[$Dow][$k]})
            {
                $m = 1;
                if (exists $Data->{$Key} and defined $Data->{$Key} and ref($Data->{$Key}) eq 'ARRAY' and @{$Data->{$Key}})
                {
                    for ( $n = 0; $n < @{$Data->{$Key}}; $n++ ) { if ($Data->{$Key}[$n][$NAME] eq $fixed) { $m = 0; last; } }
                }
                else { $Data->{$Key} = []; }
                if ($m)
                {
                    unshift( @{$Data->{$Key}}, [ $fixed, timestamp(), [] ] );
                    $Save = 1;
                }
            }
        }
    }
}

if ($Save) # if a command was executed, data structure must be stored
{
    eval { lock_nstore($Data, $File); };
}

print <<"VERBATIM";
Content-type: text/html; charset="iso-8859-1"

<HTML>
<HEAD>
    <META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=iso-8859-1">
    <TITLE>$Title</TITLE>
</HEAD>
<BODY>
<A HREF="help.html">ONLINE HELP</A><BR>
<A HREF="../download/pkg/demo.tar.gz">Download the complete software of this demo</A>
<CENTER>
    <P>
    <HR NOSHADE SIZE="2">
    <P>
    <H1>$Title</H1>
    <P>
    <HR NOSHADE SIZE="2">
    <P>
    <TABLE BGCOLOR="#F4F4F4" CELLSPACING="1" CELLPADDING="7" BORDER="2"><TR><TD>
    <FORM METHOD="GET" ACTION="">
    <INPUT TYPE="hidden" NAME="y" VALUE="$Date[0]">
    <INPUT TYPE="hidden" NAME="m" VALUE="$Date[1]">
    <INPUT TYPE="hidden" NAME="d" VALUE="$Date[2]">
    <TABLE BGCOLOR="#F4F4F4" CELLSPACING="1" CELLPADDING="7" BORDER="0">
    <TR>
    <TD>$Text[0][$Lang]</TD>
    <TD>
    <SELECT NAME="l">
VERBATIM
for ( $n = 1; $n <= 6; $n++ )
{
    if ($n == $Lang) { print qq(    <OPTION VALUE="$n" SELECTED>$n - $lang[$n]</OPTION>\n); }
    else             { print qq(    <OPTION VALUE="$n">$n - $lang[$n]</OPTION>\n); }
}
print <<"VERBATIM";
    </SELECT>
    </TD>
    <TD>
    <INPUT TYPE="submit" VALUE="$Text[1][$Lang]">
    </TD>
    </TR>
    </TABLE>
    </FORM>
    </TD></TR></TABLE>
    <P>
    <TABLE BGCOLOR="#F4F4F4" CELLSPACING="1" CELLPADDING="15" BORDER="2">
    <TR>
    <TD VALIGN="TOP">
VERBATIM

calendar(@Date,@Date);

print <<"VERBATIM";
    </TD>
    <TD VALIGN="TOP">
VERBATIM

calendar(@Date,@Next);

print <<"VERBATIM";
    </TR>
    <TR>
    <TD ALIGN="CENTER">
    <FORM METHOD="GET" ACTION="">
    <INPUT TYPE="hidden" NAME="y" VALUE="$Prev[0]">
    <INPUT TYPE="hidden" NAME="m" VALUE="$Prev[1]">
    <INPUT TYPE="hidden" NAME="d" VALUE="$Prev[2]">
    <INPUT TYPE="hidden" NAME="l" VALUE="$Lang">
    <INPUT TYPE="submit" VALUE="&nbsp;&lt;&nbsp;&lt;&nbsp;&lt;&nbsp;">
    </FORM>
    </TD>
    <TD ALIGN="CENTER">
    <FORM METHOD="GET" ACTION="">
    <INPUT TYPE="hidden" NAME="y" VALUE="$Next[0]">
    <INPUT TYPE="hidden" NAME="m" VALUE="$Next[1]">
    <INPUT TYPE="hidden" NAME="d" VALUE="$Next[2]">
    <INPUT TYPE="hidden" NAME="l" VALUE="$Lang">
    <INPUT TYPE="submit" VALUE="&nbsp;&gt;&nbsp;&gt;&nbsp;&gt;&nbsp;">
    </FORM>
    </TD>
    </TR>
    </TABLE>
    <P>
    <HR NOSHADE SIZE="2">
    <P>
    <H2>$Head</H2>
    <P>
    <HR NOSHADE SIZE="2">
    <P>
VERBATIM
    if (exists $Data->{$Key} and defined $Data->{$Key} and ref($Data->{$Key}) eq 'ARRAY' and @{$Data->{$Key}})
    {
        $n = @{$Data->{$Key}};
        if ($n == 1)
        {
            print qq(    <H3>$Text[3][$Lang]</H3>\n);
        }
        else
        {
            $m = $Text[4][$Lang];
            $m =~ s/#/$n/;
            print qq(    <H3>$m</H3>\n);
        }
    }
    else
    {
        $n = 0;
        print qq(    <H3>$Text[2][$Lang]</H3>\n);
    }
    print <<"VERBATIM";
    <P>
    <TABLE BGCOLOR="#F4F4F4" CELLSPACING="1" CELLPADDING="15" BORDER="2">
VERBATIM
    for ( $m = 0, $c = 1; $m < $n; $m++, $c++ )
    {
        $e = $Data->{$Key}[$m][$NAME];
        $z = $Data->{$Key}[$m][$TIME];
        print <<"VERBATIM";
    <TR>
    <A NAME="$c">
    <TD ALIGN="CENTER" VALIGN="middle">#$c</TD>
    </A>
    <TD ALIGN="CENTER" VALIGN="middle">
    <FORM METHOD="GET" ACTION="">
    <INPUT TYPE="hidden" NAME="y" VALUE="$Date[0]">
    <INPUT TYPE="hidden" NAME="m" VALUE="$Date[1]">
    <INPUT TYPE="hidden" NAME="d" VALUE="$Date[2]">
    <INPUT TYPE="hidden" NAME="l" VALUE="$Lang">
    <INPUT TYPE="hidden" NAME="e" VALUE="$m">
    <INPUT TYPE="hidden" NAME="c" VALUE="de">
    <INPUT TYPE="hidden" NAME="t" VALUE="$Time">
    <TEXTAREA ROWS="6" COLS="80" WRAP="physical">$e</TEXTAREA>
    <BR>
    <FONT SIZE="-2">$z</FONT>
    <P>
    <INPUT TYPE="submit" VALUE="$Text[8][$Lang]">
    </FORM>
    <P>
VERBATIM
        $p = @{$Data->{$Key}[$m][$LIST]};
        if ($p == 0)
        {
            print qq(    <B>$Text[5][$Lang]</B>\n);
        }
        else
        {
            if ($p == 1)
            {
                print qq(    <B>$Text[6][$Lang]</B>\n);
            }
            else
            {
                $e = $Text[7][$Lang];
                $e =~ s/#/$p/;
                print qq(    <B>$e</B>\n);
            }
        }
        print <<"VERBATIM";
    <P>
    <TABLE BGCOLOR="#E0E0E0" CELLSPACING="1" CELLPADDING="7" BORDER="2">
VERBATIM
        for ( $l = 0, $o = 1; $l < $p; $l++, $o++ )
        {
            $e = $Data->{$Key}[$m][$LIST][$l][$NAME];
            $z = $Data->{$Key}[$m][$LIST][$l][$TIME];
            print <<"VERBATIM";
    <TR>
    <TD ALIGN="RIGHT" VALIGN="middle">$o</TD>
    <TD ALIGN="LEFT" VALIGN="middle">$e<BR><FONT SIZE="-2">$z</FONT></TD>
    <TD ALIGN="CENTER" VALIGN="middle">
    <FORM METHOD="GET" ACTION="">
    <INPUT TYPE="hidden" NAME="y" VALUE="$Date[0]">
    <INPUT TYPE="hidden" NAME="m" VALUE="$Date[1]">
    <INPUT TYPE="hidden" NAME="d" VALUE="$Date[2]">
    <INPUT TYPE="hidden" NAME="l" VALUE="$Lang">
    <INPUT TYPE="hidden" NAME="e" VALUE="$m">
    <INPUT TYPE="hidden" NAME="p" VALUE="$l">
    <INPUT TYPE="hidden" NAME="c" VALUE="dp">
    <INPUT TYPE="hidden" NAME="t" VALUE="$Time">
    <INPUT TYPE="submit" VALUE="$Text[9][$Lang]">
    </FORM>
    </TD>
    </TR>
VERBATIM
        }
        print <<"VERBATIM";
    <TR>
    <TD COLSPAN="2" ALIGN="CENTER" VALIGN="middle">
    <FORM METHOD="GET" ACTION="">
    <INPUT TYPE="hidden" NAME="y" VALUE="$Date[0]">
    <INPUT TYPE="hidden" NAME="m" VALUE="$Date[1]">
    <INPUT TYPE="hidden" NAME="d" VALUE="$Date[2]">
    <INPUT TYPE="hidden" NAME="l" VALUE="$Lang">
    <INPUT TYPE="hidden" NAME="e" VALUE="$m">
    <INPUT TYPE="hidden" NAME="c" VALUE="am">
    <INPUT TYPE="hidden" NAME="t" VALUE="$Time">
    <SELECT NAME="p">
VERBATIM
        $p = @Members;
        for ( $l = 0; $l < $p; $l++ )
        {
            $o = $Members[$l];
            print qq(    <OPTION VALUE="$l">$o</OPTION>\n);
        }
        print <<"VERBATIM";
    </SELECT>
    </TD>
    <TD ALIGN="CENTER" VALIGN="middle">
    <INPUT TYPE="submit" VALUE="$Text[10][$Lang]">
    </FORM>
    </TD>
    </TR>
    <TR>
    <TD COLSPAN="2" ALIGN="CENTER" VALIGN="middle">
    <FORM METHOD="GET" ACTION="">
    <INPUT TYPE="hidden" NAME="y" VALUE="$Date[0]">
    <INPUT TYPE="hidden" NAME="m" VALUE="$Date[1]">
    <INPUT TYPE="hidden" NAME="d" VALUE="$Date[2]">
    <INPUT TYPE="hidden" NAME="l" VALUE="$Lang">
    <INPUT TYPE="hidden" NAME="e" VALUE="$m">
    <INPUT TYPE="hidden" NAME="c" VALUE="ap">
    <INPUT TYPE="hidden" NAME="t" VALUE="$Time">
    <INPUT TYPE="text"   NAME="p" VALUE="" SIZE="40" MAXLENGTH="80">
    </TD>
    <TD ALIGN="CENTER" VALIGN="middle">
    <INPUT TYPE="submit" VALUE="$Text[11][$Lang]">
    </FORM>
    </TD>
    </TR>
    </TABLE>
    </TD>
    </TR>
VERBATIM
    }
    print <<"VERBATIM";
    <TR>
    <A NAME="$c">
    <TD ALIGN="CENTER" VALIGN="middle">#$c</TD>
    </A>
    <TD ALIGN="CENTER" VALIGN="middle">
    <FORM METHOD="GET" ACTION="">
    <INPUT TYPE="hidden" NAME="y" VALUE="$Date[0]">
    <INPUT TYPE="hidden" NAME="m" VALUE="$Date[1]">
    <INPUT TYPE="hidden" NAME="d" VALUE="$Date[2]">
    <INPUT TYPE="hidden" NAME="l" VALUE="$Lang">
    <INPUT TYPE="hidden" NAME="c" VALUE="ae">
    <INPUT TYPE="hidden" NAME="t" VALUE="$Time">
    <TEXTAREA NAME="e" ROWS="6" COLS="80" WRAP="physical"></TEXTAREA>
    <P>
    <INPUT TYPE="submit" VALUE="$Text[12][$Lang]">
    </FORM>
    </TD>
    </TR>
    </TABLE>
    <P>
    <HR NOSHADE SIZE="2">
    <P>
</CENTER>
<A HREF="../download/pkg/demo.tar.gz">Download the complete software of this demo</A><BR>
<A HREF="help.html">ONLINE HELP</A>
</BODY>
</HTML>
VERBATIM

sub process_query_string
{
    my $query = $ENV{'QUERY_STRING'} || $ENV{'REDIRECT_QUERY_STRING'} || join('&', @ARGV) || '';
    my @pairs = split(/&/, $query);
    my($pair,$var,$val);

    foreach $pair (@pairs)
    {
        ($var,$val) = split(/=/,$pair,2);
        if ($var =~ /^[a-z]+$/i)
        {
            $val =~ s/\+/ /g;
            $val =~ s/%([0-9A-Fa-f][0-9A-Fa-f])/chr(hex($1))/eg;
            $val =~ s/^\s+//;
            $val =~ s/\s+$//;
            if    ($var =~ /^y(?:ear)?$/i)
            {
                if ($val =~ /^\d+$/ and $val >= 1970 and $val < 2038) { $Date[0] = $val; }
            }
            elsif ($var =~ /^m(?:onth)?$/i)
            {
                if ($val =~ /^\d+$/ and $val >= 1 and $val <= 12) { $Date[1] = $val; }
            }
            elsif ($var =~ /^d(?:ay)?$/i)
            {
                if ($val =~ /^\d+$/ and $val >= 1 and $val <= 31) { $Date[2] = $val; }
            }
            elsif ($var =~ /^l(?:ang(?:uage)?)?$/i)
            {
                if (($val =~ /^\d+$/ or ($val = Decode_Language($val))) and $val >= 1 and $val <= 6) { $Lang = $val; }
            }
            elsif ($var =~ /^e(?:vt|vent)?$/i)
            {
                if ($val =~ /^\d+$/) { $Event = $val; } else { $Event = html($val); }
            }
            elsif ($var =~ /^p(?:articipant)?$/i)
            {
                if ($val =~ /^\d+$/) { $Person = $val; } else { $Person = html($val); }
            }
            elsif ($var =~ /^c(?:md|ommand)?$/i)
            {
                if ($val =~ /^(?:ae|am|ap|de|dp)$/i) { $Command = $val; }
            }
            elsif ($var =~ /^t(?:ime(?:out)?)?$/i)
            {
                if ($val =~ /^\d+$/) { $Timeout = $val; }
            }
        }
    }
    unless (check_date(@Date))
    {
        @Date = @Today;
        $Command = '';
    }
    if (($Timeout < ($Time - 120)) or
        ($Timeout >= $Time)        or
        (($Command =~ /^am$/i) and ($Timeout < $Updated)))
    {
        $Command = '';
    }
}

sub execute_command
{
    if ($Command =~ /^ae$/i)
    {
        if ($Event ne '')
        {
            $Data->{$Key} = [] unless (exists $Data->{$Key} and defined $Data->{$Key} and ref($Data->{$Key}) eq 'ARRAY');
            push( @{$Data->{$Key}}, [ $Event, timestamp(), [] ] );
            $Save = 1;
        }
    }
    elsif (exists $Data->{$Key} and defined $Data->{$Key} and ref($Data->{$Key}) eq 'ARRAY' and @{$Data->{$Key}})
    {
        if    ($Command =~ /^am$/i)
        {
            if ($Event  =~ /^\d+$/ and $Event  < @{$Data->{$Key}} and
                $Person =~ /^\d+$/ and $Person > 0 and $Person < @Members)
            {
                push( @{$Data->{$Key}[$Event][$LIST]}, [ $Members[$Person], timestamp() ] );
                $Save = 1;
            }
        }
        elsif ($Command =~ /^ap$/i)
        {
            if ($Event =~ /^\d+$/ and $Event < @{$Data->{$Key}} and $Person ne '')
            {
                push( @{$Data->{$Key}[$Event][$LIST]}, [ $Person, timestamp() ] );
                $Save = 1;
            }
        }
        elsif ($Command =~ /^de$/i)
        {
            if ($Event =~ /^\d+$/ and $Event < @{$Data->{$Key}})
            {
                splice( @{$Data->{$Key}}, $Event, 1 );
                delete $Data->{$Key} unless (@{$Data->{$Key}});
                $Save = 1;
            }
        }
        elsif ($Command =~ /^dp$/i)
        {
            if ($Event  =~ /^\d+$/ and $Event  < @{$Data->{$Key}} and
                $Person =~ /^\d+$/ and $Person < @{$Data->{$Key}[$Event][$LIST]})
            {
                splice( @{$Data->{$Key}[$Event][$LIST]}, $Person, 1 );
                $Save = 1;
            }
        }
    }
}

sub calendar
{
    my($Y,$M,$D,$yy,$mm,$dd) = @_;
    my($len) = Days_in_Month($yy,$mm);
    my($dow) = Day_of_Week($yy,$mm,1) - 1;
    my(@tag) = (('&nbsp;') x (7*6));
    my(@hdr,@lbl,$b,$i,$j,$k,$n,$t);

    $t = Month_to_Text($mm,$Lang);
    $j = substr($t,0,1);
    substr($t,0,1) = ISO_UC($j);
    $hdr[0] = html($t);
    $hdr[8] = (qw(Error Week Semaine KW Semana Semana Week))[$Lang];
    for ( $i = 1; $i < 8; $i++ ) { $hdr[$i] = html(Day_of_Week_Abbreviation($i,$Lang)); }
    for ( $i = 0; $i < 7; $i++ ) { $lbl[$i] = Week_of_Year(Add_Delta_Days($yy,$mm,1,$i*7)); }
    for ( $i = $dow, $j = 1, $k = Date_to_Days($yy,$mm,1); $j <= $len; $i++, $j++, $k++ )
    {
        unless ($b = (exists $Data->{$k} and defined $Data->{$k} and ref($Data->{$k}) eq 'ARRAY' and @{$Data->{$k}}))
        {
            $t = ($i % 7) + 1;
            foreach $n (nth_dow($t,$yy,$mm,$j))
            {
                if (defined $Fixed[$t][$n] and ref($Fixed[$t][$n]) eq 'ARRAY') { $b = 1; last; }
            }
        }
        if ($b)
        {
            $t = "<B>$j</B>";
        }
        else { $t = "$j"; }
        if ($j == $D and $mm == $M and $yy == $Y) { $tag[$i] = qq(<FONT COLOR="#FF0000">$t</FONT>); }
        else                                      { $tag[$i] = qq(<A HREF="?y=$yy&m=$mm&d=$j&l=$Lang">$t</A>); }
    }
    print <<"VERBATIM";
    <TABLE BGCOLOR="#E0E0E0" CELLSPACING="1" CELLPADDING="7" BORDER="2">
    <TR>
        <TD COLSPAN="8" ALIGN="CENTER">$hdr[0] $yy</TD>
    </TR>
    <TR>
        <TD BGCOLOR="#EEEEEE" WIDTH="16%" ALIGN="CENTER">$hdr[8]</TD>
        <TD WIDTH="12%" ALIGN="RIGHT">$hdr[1]</TD>
        <TD WIDTH="12%" ALIGN="RIGHT">$hdr[2]</TD>
        <TD WIDTH="12%" ALIGN="RIGHT">$hdr[3]</TD>
        <TD WIDTH="12%" ALIGN="RIGHT">$hdr[4]</TD>
        <TD WIDTH="12%" ALIGN="RIGHT">$hdr[5]</TD>
        <TD WIDTH="12%" ALIGN="RIGHT">$hdr[6]</TD>
        <TD WIDTH="12%" ALIGN="RIGHT">$hdr[7]</TD>
    </TR>
    <TR>
        <TD BGCOLOR="#EEEEEE" ALIGN="CENTER">$lbl[0]</TD>
        <TD ALIGN="RIGHT">$tag[0]</TD>
        <TD ALIGN="RIGHT">$tag[1]</TD>
        <TD ALIGN="RIGHT">$tag[2]</TD>
        <TD ALIGN="RIGHT">$tag[3]</TD>
        <TD ALIGN="RIGHT">$tag[4]</TD>
        <TD ALIGN="RIGHT">$tag[5]</TD>
        <TD ALIGN="RIGHT">$tag[6]</TD>
    </TR>
    <TR>
        <TD BGCOLOR="#EEEEEE" ALIGN="CENTER">$lbl[1]</TD>
        <TD ALIGN="RIGHT">$tag[7]</TD>
        <TD ALIGN="RIGHT">$tag[8]</TD>
        <TD ALIGN="RIGHT">$tag[9]</TD>
        <TD ALIGN="RIGHT">$tag[10]</TD>
        <TD ALIGN="RIGHT">$tag[11]</TD>
        <TD ALIGN="RIGHT">$tag[12]</TD>
        <TD ALIGN="RIGHT">$tag[13]</TD>
    </TR>
    <TR>
        <TD BGCOLOR="#EEEEEE" ALIGN="CENTER">$lbl[2]</TD>
        <TD ALIGN="RIGHT">$tag[14]</TD>
        <TD ALIGN="RIGHT">$tag[15]</TD>
        <TD ALIGN="RIGHT">$tag[16]</TD>
        <TD ALIGN="RIGHT">$tag[17]</TD>
        <TD ALIGN="RIGHT">$tag[18]</TD>
        <TD ALIGN="RIGHT">$tag[19]</TD>
        <TD ALIGN="RIGHT">$tag[20]</TD>
    </TR>
    <TR>
        <TD BGCOLOR="#EEEEEE" ALIGN="CENTER">$lbl[3]</TD>
        <TD ALIGN="RIGHT">$tag[21]</TD>
        <TD ALIGN="RIGHT">$tag[22]</TD>
        <TD ALIGN="RIGHT">$tag[23]</TD>
        <TD ALIGN="RIGHT">$tag[24]</TD>
        <TD ALIGN="RIGHT">$tag[25]</TD>
        <TD ALIGN="RIGHT">$tag[26]</TD>
        <TD ALIGN="RIGHT">$tag[27]</TD>
    </TR>
VERBATIM
    if ($i > 28)
    {
        print <<"VERBATIM";
    <TR>
        <TD BGCOLOR="#EEEEEE" ALIGN="CENTER">$lbl[4]</TD>
        <TD ALIGN="RIGHT">$tag[28]</TD>
        <TD ALIGN="RIGHT">$tag[29]</TD>
        <TD ALIGN="RIGHT">$tag[30]</TD>
        <TD ALIGN="RIGHT">$tag[31]</TD>
        <TD ALIGN="RIGHT">$tag[32]</TD>
        <TD ALIGN="RIGHT">$tag[33]</TD>
        <TD ALIGN="RIGHT">$tag[34]</TD>
    </TR>
VERBATIM
    }
    if ($i > 35)
    {
        print <<"VERBATIM";
    <TR>
        <TD BGCOLOR="#EEEEEE" ALIGN="CENTER">$lbl[5]</TD>
        <TD ALIGN="RIGHT">$tag[35]</TD>
        <TD ALIGN="RIGHT">$tag[36]</TD>
        <TD ALIGN="RIGHT">$tag[37]</TD>
        <TD ALIGN="RIGHT">$tag[38]</TD>
        <TD ALIGN="RIGHT">$tag[39]</TD>
        <TD ALIGN="RIGHT">$tag[40]</TD>
        <TD ALIGN="RIGHT">$tag[41]</TD>
    </TR>
VERBATIM
    }
    print <<"VERBATIM";
    </TABLE>
VERBATIM
}

sub html
{
    my($string) = $_[0];
    my($o);
    $string =~ s/(.)/(defined $html[$o=ord($1)])?($html[$o]||"&\#$o;"):$1/eg;
    return $string;
}

sub init_tables
{
    my($i);
    $html[0x00] = '';
    $html[0x01] = '';
    $html[0x02] = '';
    $html[0x03] = '';
    $html[0x04] = '';
    $html[0x05] = '';
    $html[0x06] = '';
    $html[0x07] = '';
    $html[0x08] = '';
    # $html[0x09] = '';
    # $html[0x0A] = '';
    $html[0x0B] = '';
    $html[0x0C] = '';
    $html[0x0D] = '';
    $html[0x0E] = '';
    $html[0x0F] = '';
    $html[0x10] = '';
    $html[0x11] = '';
    $html[0x12] = '';
    $html[0x13] = '';
    $html[0x14] = '';
    $html[0x15] = '';
    $html[0x16] = '';
    $html[0x17] = '';
    $html[0x18] = '';
    $html[0x19] = '';
    $html[0x1A] = '';
    $html[0x1B] = '';
    $html[0x1C] = '';
    $html[0x1D] = '';
    $html[0x1E] = '';
    $html[0x1F] = '';
    $html[0x22] = '&quot;';
    $html[0x26] = '&amp;';
    # $html[0x27] = '&apos;';
    $html[0x3C] = '&lt;';
    $html[0x3E] = '&gt;';
    $html[0x7F] = '';
    $html[0x80] = '';
    $html[0x81] = '';
    $html[0x82] = '';
    $html[0x83] = '';
    $html[0x84] = '';
    $html[0x85] = '';
    $html[0x86] = '';
    $html[0x87] = '';
    $html[0x88] = '';
    $html[0x89] = '';
    $html[0x8A] = '';
    $html[0x8B] = '';
    $html[0x8C] = '';
    $html[0x8D] = '';
    $html[0x8E] = '';
    $html[0x8F] = '';
    $html[0x90] = '';
    $html[0x91] = '';
    $html[0x92] = '';
    $html[0x93] = '';
    $html[0x94] = '';
    $html[0x95] = '';
    $html[0x96] = '';
    $html[0x97] = '';
    $html[0x98] = '';
    $html[0x99] = '';
    $html[0x9A] = '';
    $html[0x9B] = '';
    $html[0x9C] = '';
    $html[0x9D] = '';
    $html[0x9E] = '';
    $html[0x9F] = '';
    $html[0xA0] = '&nbsp;';
    $html[0xA1] = '&iexcl;';
    $html[0xA2] = '&cent;';
    $html[0xA3] = '&pound;';
    $html[0xA4] = '&curren;';
    $html[0xA5] = '&yen;';
    $html[0xA6] = '&brvbar;';
    $html[0xA7] = '&sect;';
    $html[0xA8] = '&uml;';
    $html[0xA9] = '&copy;';
    $html[0xAA] = '&ordf;';
    $html[0xAB] = '&laquo;';
    $html[0xAC] = '&not;';
    $html[0xAD] = '&shy;';
    $html[0xAE] = '&reg;';
    $html[0xAF] = '&macr;';
    $html[0xB0] = '&deg;';
    $html[0xB1] = '&plusmn;';
    $html[0xB2] = '&sup2;';
    $html[0xB3] = '&sup3;';
    $html[0xB4] = '&acute;';
    $html[0xB5] = '&micro;';
    $html[0xB6] = '&para;';
    $html[0xB7] = '&middot;';
    $html[0xB8] = '&cedil;';
    $html[0xB9] = '&sup1;';
    $html[0xBA] = '&ordm;';
    $html[0xBB] = '&raquo;';
    $html[0xBC] = '&frac14;';
    $html[0xBD] = '&frac12;';
    $html[0xBE] = '&frac34;';
    $html[0xBF] = '&iquest;';
    $html[0xC0] = '&Agrave;';
    $html[0xC1] = '&Aacute;';
    $html[0xC2] = '&Acirc;';
    $html[0xC3] = '&Atilde;';
    $html[0xC4] = '&Auml;';
    $html[0xC5] = '&Aring;';
    $html[0xC6] = '&AElig;';
    $html[0xC7] = '&Ccedil;';
    $html[0xC8] = '&Egrave;';
    $html[0xC9] = '&Eacute;';
    $html[0xCA] = '&Ecirc;';
    $html[0xCB] = '&Euml;';
    $html[0xCC] = '&Igrave;';
    $html[0xCD] = '&Iacute;';
    $html[0xCE] = '&Icirc;';
    $html[0xCF] = '&Iuml;';
    $html[0xD0] = '&ETH;';
    $html[0xD1] = '&Ntilde;';
    $html[0xD2] = '&Ograve;';
    $html[0xD3] = '&Oacute;';
    $html[0xD4] = '&Ocirc;';
    $html[0xD5] = '&Otilde;';
    $html[0xD6] = '&Ouml;';
    $html[0xD7] = '&times;';
    $html[0xD8] = '&Oslash;';
    $html[0xD9] = '&Ugrave;';
    $html[0xDA] = '&Uacute;';
    $html[0xDB] = '&Ucirc;';
    $html[0xDC] = '&Uuml;';
    $html[0xDD] = '&Yacute;';
    $html[0xDE] = '&THORN;';
    $html[0xDF] = '&szlig;';
    $html[0xE0] = '&agrave;';
    $html[0xE1] = '&aacute;';
    $html[0xE2] = '&acirc;';
    $html[0xE3] = '&atilde;';
    $html[0xE4] = '&auml;';
    $html[0xE5] = '&aring;';
    $html[0xE6] = '&aelig;';
    $html[0xE7] = '&ccedil;';
    $html[0xE8] = '&egrave;';
    $html[0xE9] = '&eacute;';
    $html[0xEA] = '&ecirc;';
    $html[0xEB] = '&euml;';
    $html[0xEC] = '&igrave;';
    $html[0xED] = '&iacute;';
    $html[0xEE] = '&icirc;';
    $html[0xEF] = '&iuml;';
    $html[0xF0] = '&eth;';
    $html[0xF1] = '&ntilde;';
    $html[0xF2] = '&ograve;';
    $html[0xF3] = '&oacute;';
    $html[0xF4] = '&ocirc;';
    $html[0xF5] = '&otilde;';
    $html[0xF6] = '&ouml;';
    $html[0xF7] = '&divide;';
    $html[0xF8] = '&oslash;';
    $html[0xF9] = '&ugrave;';
    $html[0xFA] = '&uacute;';
    $html[0xFB] = '&ucirc;';
    $html[0xFC] = '&uuml;';
    $html[0xFD] = '&yacute;';
    $html[0xFE] = '&thorn;';
    $html[0xFF] = '&yuml;';
    for ( $i = 1; $i <= 6; $i++ ) { $lang[$i] = html(Language_to_Text($i)); }
}

sub timestamp
{
    my(@now) = Today_and_Now();
    return sprintf("%02d-%02d-%04d %02d:%02d:%02d", @now[2,1,0,3,4,5]);
}

sub nth_dow
{
    my($dow) = shift;
    my($n) = int(Delta_Days(Nth_Weekday_of_Month_Year(@_[0,1],$dow,1),@_)/7)+1;
    if (($n == 4) and ((Add_Delta_Days(@_,7))[1] != $_[1])) { return (0,4,5); }
    else                                                    { return (0,$n);  }
}

__END__

